/*++

Copyright (c) Sydus.

Module:

    Hmac.h

Authors:

    Sharvil Nanavati (sharvil) 2007-05-16

--*/

#pragma once

#include "base.h"

template <typename T>
class Hmac
{
	public:
		Hmac();
		~Hmac();

		BString Mac(const BString & key, const BString & message);

	private:
		static const uint8 KOuterPadBitPattern = 0x5C;
		static const uint8 KInnerPadBitPattern = 0x36;
};

template <typename T>
Hmac <T>::Hmac()
{
}

template <typename T>
Hmac <T>::~Hmac()
{
}

template <typename T>
BString Hmac <T>::Mac(const BString & key, const BString & message)
{
	uint8 innerPad[T::KBlockSize];
	uint8 outerPad[T::KBlockSize];

	BString actualKey;
	if(key.GetLength() > T::KBlockSize)
		actualKey = T().Hash(key);
	else
		actualKey = key;

	for(uint32 i = 0; i < actualKey.GetLength(); ++i)
	{
		outerPad[i] = KOuterPadBitPattern ^ actualKey[i];
		innerPad[i] = KInnerPadBitPattern ^ actualKey[i];
	}
	for(uint32 i = actualKey.GetLength(); i < T::KBlockSize; ++i)
	{
		outerPad[i] = KOuterPadBitPattern;
		innerPad[i] = KInnerPadBitPattern;
	}

	T innerHash;
	innerHash << BString(innerPad, T::KBlockSize) << message;

	T outerHash;
	outerHash << BString(outerPad, T::KBlockSize) << innerHash.Hash();

	return outerHash.Hash();
}
