/*++

Copyright (c) Sydus.

Module:

    InetAddress.h

Authors:

    Sharvil Nanavati (sharvil) 2005-01-08

--*/

#pragma once

#include "base.h"

struct sockaddr_in;
class InetAddress
{
	private:
		static const uint32 cInvalidPort;

	public:
		InetAddress();
		InetAddress(const std::string & address, uint16 portInHostOrder);
		InetAddress(uint32 ipAddressInHostOrder, uint16 portInHostOrder);
		explicit InetAddress(const std::string & address);
		explicit InetAddress(const sockaddr_in & address);
		explicit InetAddress(uint32 ipAddressInHostOrder);
		InetAddress(const InetAddress & address);

		std::string GetIpAddress() const;
		uint16 GetPort() const;

		std::string AsString() const;
		uint32 AsUint32NetworkOrder() const;
		uint32 AsUint32HostOrder() const;
		sockaddr_in AsSockaddr() const;

		InetAddress & operator = (const InetAddress & address);
		bool operator == (const InetAddress & address) const;
		bool operator != (const InetAddress & address) const;
		bool operator <  (const InetAddress & address) const;
		operator std::string () const;

	private:
		union
		{
			uint8 iIpBytes[4];
			uint32 iIpNetworkOrder;
		};
		uint32 iPort;
};
