/*++

Copyright (c) Sydus.

Module:

    Socket_Posix.h

Authors:

    Sharvil Nanavati (sharvil) 2005-08-25
    Emily Johnston (emily) 2005-12-23

--*/

#pragma once

#include "base.h"
#include "ISocketImplementation.h"
#include "InetAddress.h"
#include "Socket.h"

class Socket_Posix : public ISocketImplementation
{
	public:
		explicit Socket_Posix(const ENetworkProtocol & protocol);
		~Socket_Posix();

		ISocketImplementation * Accept();
		void Bind(const InetAddress & ip, uint16 port);
		void Listen();

		void Connect(const std::string & host, uint16 port);
		void Close();

		uint32 Read(void * buffer, uint32 size) const;
		uint32 Write(const void * buffer, uint32 size) const;

		uint32 Read(InetAddress & source, void * buffer, uint32 size) const;
		uint32 Write(const InetAddress & destination, const void * buffer, uint32 size) const;

		bool IsReadable() const;
		bool IsWritable() const;

		const InetAddress & GetRemoteAddress() const;

		void SetReuseAddress(bool reuse);
		void SetBlocking();
		void SetNonBlocking();
		void EnableKeepAlive(int32 idleTime = -1, int32 interval = -1, int32 numProbes = -1);
		void DisableKeepAlive();
		void EnableNagle();
		void DisableNagle();

	private:
		Socket_Posix(int32 fileDescriptor, const InetAddress & host);
		int32 GetFd() const;

		int32 iFd;
		InetAddress iRemoteHost;
};
