/*++

Copyright (c) Sydus.

Module:

    SSDispatcher.h

Authors:

    Sharvil Nanavati (sharvil) 2008-06-17

--*/

#pragma once

#include "base.h"
#include "ISSProtocol.h"

#include <map>

struct SSNullProtocol
{
	static ISSProtocol * Create() { return 0; }
};

template <typename Factory>
class SSDispatcher
{
	public:
		explicit SSDispatcher(SSConnectionPool & pool);
		~SSDispatcher();

		void Poll();

	private:
		typedef std::map <InetAddress, ISSProtocol *> HandlerMap;

		SSConnectionPool & iPool;
		HandlerMap iHandlers;
};

template <typename T>
SSDispatcher <T>::SSDispatcher(SSConnectionPool & pool)
	: iPool(pool)
{
}

template <typename T>
SSDispatcher <T>::~SSDispatcher()
{
	for(HandlerMap::iterator i = iHandlers.begin(); i != iHandlers.end(); ++i)
		delete i->second;
}

template <typename T>
void SSDispatcher <T>::Poll()
{
	iPool.Poll();

	//
	// Handle existing connections and delete handlers for connections
	// that have died.
	//
	for(HandlerMap::iterator i = iHandlers.begin(); i != iHandlers.end(); )
	{
		SSConnection * connection = iPool.GetConnection(i->first);
		if(connection == 0)
		{
			i->second->OnDisconnect();
			delete i->second;

			HandlerMap::iterator tmp = i++;
			iHandlers.erase(tmp);
			continue;
		}

		while(connection->HasMorePackets())
			i->second->OnReceive(*connection, connection->GetNextPacket());

		i->second->Poll(*connection);
		++i;
	}

	//
	// Create new handlers for incoming connections.
	//
	for(SSConnectionPool::iterator i = iPool.begin(); i != iPool.end(); ++i)
	{
		HandlerMap::iterator j = iHandlers.find(i->first);
		if(j == iHandlers.end())
		{
			ISSProtocol * p = T::Create();
			if(p == 0)
				continue;

			j = iHandlers.insert(std::make_pair(i->first, p)).first;
			j->second->OnConnect(*i->second);
		}

		while(i->second->HasMorePackets())
			j->second->OnReceive(*i->second, i->second->GetNextPacket());
	}
}
