/*++

Copyright (c) Sharvil Nanavati.

Module:

    ZoneList.h

Authors:

    Sharvil Nanavati (sharvil) 2008-04-27

--*/

#pragma once

#include "base.h"
#include "SSNetLib.h"
#include "TimeLib.h"

#include <vector>
#include <list>

class Logger;
class Config;

struct Zone
{
	bool iIsMaster;
	bool iIsDisabled;             // Whether a master is disabled (too long since last update)
	bool iIsUpdating;             // Whether an update is pending for this sweep

	uint32 iUpdateTime;

	InetAddress iAddress;
	std::string iName;
	std::string iPassword;
	std::string iDescription;

	uint32 iServerVersion;
	uint16 iPlayerCount;
	bool iIsBilled;

	uint16 iMinPlayerCount;
	uint16 iMaxPlayerCount;
};

class ZoneList
{
	private:
		typedef std::vector <InetAddress> Sources;
		typedef std::list <Zone> Zones;

	public:
		typedef Zones::const_iterator iterator;
		typedef Zones::const_iterator const_iterator;

		explicit ZoneList(const Config & config);
		~ZoneList();

		void AddSource(const InetAddress & address);
		void RemoveSource(const InetAddress & address);

		void ServerUpdate(InetAddress & source, const BString & packet);
		void ClientRequest(SSConnection & connection, const BString & packet);
		void WriteStatistics(const Time & serverLaunchTime);

		void Poll();

	private:
		void UpdateMasterZone(const Zone & zone);
		void UpdateMirroredZone(const Zone & zone);

		void DisableMissingMasters(uint32 currentTime);
		void ClearMirroredZones();

		void UpdateZoneLists();
		void ProcessZoneListPacket(const std::string & connName, const BString & packet);

		Logger & iLogger;
		const Config & iConfig;

		SSConnectionPool iPool;
		Sources iSources;
		Zones iZones;

		Stopwatch iZoneQueryTimer;
};
